// DacsClient.cpp: implementation of the DacsClient class
// Manages initialisation of DACS Interfaces and processes
// the AuthorizationAgentEvents
//
// Disclaimer: This sample code has been provided for teaching
// and training purposes only. It is not intended for use
// within production applications.

#include "DacsClient.h"
#include "Dacs/AuthAgentEventStatus.h"
#include "Dacs/AuthCheckStatus.h"
#include "DacsAppClient.h"
#include "stdio.h"

#include <iostream>
#include <string>

#include "basicConsumer.h"

DacsClient::DacsClient(rfa::common::EventQueue &eventQ,
					  rfa::common::StandardPrincipalIdentity &spi, rfa::common::RFA_String& dacsConnectionStr):
					   _pEventQ(eventQ), _spId(spi), _authAgent(0), _authSys(0), _loginStatus(LoginPending), _dacsConnectionStr(dacsConnectionStr)
					   
{
	
}

DacsClient::~DacsClient()
{
	cleanup();
}

int DacsClient::init()
{	
	int ret = DACSAUTH_SUCCESS;
	try{
		std::cout << std::endl << "*** Creating AuthorizationSystem and AuthorizationAgent ***\n";

		if (_dacsConnectionStr.compareIgnoreCase(rfa::common::RFA_String("local", 0))==0){
			_authSys = rfa::dacs::AuthorizationSystem::acquire("DACS");
		}
		else
		{
			std::cout << std::endl << "Use remote DACS: "<<_dacsConnectionStr.c_str()<<"\n";
			optionsElement.sOptionName.append("hosts");
			optionsElement.sOptionValue.append(_dacsConnectionStr);
			options.push_back(optionsElement);
			_authSys = rfa::dacs::AuthorizationSystem::acquire("DACSMC", &options);
		}	

		
		_authAgent = _authSys->createAuthorizationAgent("agent",true);		

		
		// Create AuthorizationRequest
		// AuthorizationRequest is derived from rfa::common::InterestSpec
		rfa::dacs::AuthorizationRequest* _authReq = rfa::dacs::AuthorizationRequest::create();

		// Set an appropriate usage logging level for a login request.
		rfa::dacs::AuthorizationRequest::PerformUsage usage =
					rfa::dacs::AuthorizationRequest::OnlyDenialPerformUsageLogging;
		_authReq->setUsageLogging(usage);

		// Set DACS user details for login AuthorizationRequest.
		_authReq->setPrincipalIdentity(_spId);

		std::cout << "Performing login with the following credentials:\n";
		std::cout << "\tUser:\t\t" << _spId.getName().c_str() << std::endl;
		std::cout << "\tPosition:\t" << _spId.getPosition().c_str() << std::endl;
		std::cout << "\tApplication Id:\t" << _spId.getAppName().c_str() << "\n\n";

		// Perform login with appropriate parameters.
		_phLogin = _authAgent->login(_pEventQ, *_authReq, *this, 0);

		delete _authReq;
		
	}
	catch( rfa::dacs::AuthorizationException& ae )
	{
		std::cerr << "AuthorizationException " << std::endl;
		std::cerr << "StatusText : " << ae.getStatus()->getStatusText().c_str() << std::endl;		
		ret = DACSAUTH_ERROR;
	}
	
	return ret;
}
rfa::dacs::AuthorizationAgent::AuthorizationConnection DacsClient::ConnectionStatus()
{
	if(_authAgent)
	{
		return _authAgent->daemonConnectionState();
	}
	return rfa::dacs::AuthorizationAgent::authorizationConnectionDown;
}
void DacsClient::cleanup()
{
	try{		
		std::cout << std::endl << "*** Logout ***"<<std::endl;
		if(_phLogin)
			_authAgent->logout(*_phLogin);		

		std::cout << "*** Destroying AuthorizationAgent and releasing AuthorizationSystem  ***\n";
		if(_authAgent)
		{
			_authAgent->destroy();
			_authAgent = 0;
		}

		if(_authSys)
		{
			_authSys->release();
			_authSys = 0;
		}
	}
	catch( rfa::dacs::AuthorizationException& ae)
	{
		std::cerr << "AuthorizationException " << std::endl;
		std::cerr << "StatusText : " << ae.getStatus()->getStatusText().c_str() << std::endl;
	}
}

void DacsClient::processEvent( const rfa::common::Event& event )
{
	switch(event.getType()){
		case RFA_DACS_PACKAGE :
			{
				const rfa::dacs::AuthorizationAgentEvent& AAEvent =
					static_cast<const rfa::dacs::AuthorizationAgentEvent&>(event);
				processAuthorizationEvent(AAEvent);
			}
			break;
		case rfa::common::ComplEventEnum :
			std::cout << std::endl << "Received Completion Event...";
			break;
		default:
			std::cout << std::endl << "Unrecognised eventType : " << event.getType() << std::endl;
			break;
	};


}
void DacsClient::printStatus(rfa::dacs::AuthorizationAgentEventStatus& status)
{
	std::cout << std::endl <<"\tState:\t\t";
	switch(status.getState())
	{
        case rfa::dacs::AuthorizationAgentEventStatus::LoggedIn :
			std::cout << "LoggedIn" << std::endl;			
            break;
        case rfa::dacs::AuthorizationAgentEventStatus::LoggedOut :
			std::cout << "LoggedOut" << std::endl;			
			break;
		default :
			std::cerr << "Unrecognised AuthorizationAgentEventStatus::Status" <<
				"!= LoggedIn or LoggedOut" << std::endl;
			break;
	}

	std::cout << "\tStatus Code:\t";

	switch(status.getStatusCode())
	{
		case rfa::dacs::AuthorizationAgentEventStatus::LoginDenied :
			std::cout << "LoginDenied" << std::endl;
			break;
		case rfa::dacs::AuthorizationAgentEventStatus::LogoutForced :
			std::cout << "LogoutForced" << std::endl;
			break;
		case rfa::dacs::AuthorizationAgentEventStatus::NoStatusCode :
			std::cout << "NoStatusCode" << std::endl;
			break;
		case rfa::dacs::AuthorizationAgentEventStatus::DoRepermission :
			std::cout << "DoRepermission" << std::endl;		
			break;
		case rfa::dacs::AuthorizationAgentEventStatus::AuthorizationSystemNotAvailable :
			std::cout << "AuthorizationSystemNotAvailable" << std::endl;
			break;
		default:
			std::cerr << "Unrecognised AuthorizationAgentEventStatus::StatusCode"
				<< status.getStatusCode() << std::endl;
			break;
	};
	std::cout << "\tStatus Text:\t" << status.getStatusText().c_str() << std::endl;

}

void DacsClient::processAuthorizationEvent
				( const rfa::dacs::AuthorizationAgentEvent& authAgentEvent )
{
	std::cout << std::endl <<"Received DACS Authorization Event..." << std::endl;

	
	// Extract Status from returned event
	rfa::dacs::AuthorizationAgentEventStatus status = authAgentEvent.getStatus();
	const rfa::dacs::AuthorizationRequest& authReq =
		static_cast<const rfa::dacs::AuthorizationRequest&>(authAgentEvent.getInterestSpec());

	// Extract StandardPrincipalIdentity from returned event
	const rfa::common::StandardPrincipalIdentity& spi = authReq.getPrincipalIdentity();
	std::cout << "User: " << spi.getName().c_str() << " [Position: " << spi.getPosition().c_str() <<
		" | AppId: " << spi.getAppName().c_str() << "]" << std::endl;

	printStatus(status);

	if(status.getState() == rfa::dacs::AuthorizationAgentEventStatus::LoggedIn)
	{
		if(_loginStatus == LoginPending)
			{
				_loginStatus = LoginSucceeded;
				processDacsLogin(this);
			}
	}else if (status.getState() == rfa::dacs::AuthorizationAgentEventStatus::LoggedOut)
	{
		if(_loginStatus == LoginPending || _loginStatus == LoginSucceeded)
			{
				_loginStatus = LoginFailed;
				processDacsLogout(this);
			}
	}

	if(status.getStatusCode() == rfa::dacs::AuthorizationAgentEventStatus::DoRepermission)
	{
		if(_loginStatus == LoginSucceeded)
			{
				processReperm(this);
			}
	}	
	
	// Has the event stream been closed
	if( authAgentEvent.isEventStreamClosed() ){
		std::cout << "DACS Event stream closed" << std::endl;
		_phLogin = 0;			
		if (_loginStatus != LoginFailed)
		{
			_loginStatus = LoginFailed;
			processDacsLogout(this);
		}
	}	
}

bool DacsClient::checkSubSBE(const rfa::common::RFA_String& serviceName, const rfa::common::RFA_String& itemName)
{
    bool status=false;
    
	if(_loginStatus == LoginSucceeded)
	{
		try{
			// Set an appropriate usage logging level for a check subscription.
			rfa::dacs::AuthorizationRequest::PerformUsage usage = 
				rfa::dacs::AuthorizationRequest::OnlyDenialPerformUsageLogging;

			// Will contain status information returned from SBE check
			rfa::dacs::AuthorizationCheckStatus subStatus;
            
			std::cout << std::endl << "Performing SBE check with the following credentials:\n";
			std::cout << "\tUser:\t\t" << _spId.getName().c_str() << std::endl;
			std::cout << "\tService:\t" << serviceName.c_str() << std::endl;
			std::cout << "\tItem:\t\t" << itemName.c_str() << std::endl;

			// Check User is permissioned for service/item
			rfa::dacs::AuthorizationAgent::AuthorizationCheckResult result = 
				_authAgent->checkSubscription(*_phLogin, usage, subStatus, serviceName.c_str(), itemName.c_str());

			// Report status of checkSubscription() call
			std::cout << "\nSBE check result...\n";
			std::cout << "\tStatus Code:\t";


			switch( result )
			{
				case rfa::dacs::AuthorizationAgent::accessAllowed :
					std::cout << "accessAllowed";
					status=true;
					break;
				case rfa::dacs::AuthorizationAgent::accessDenied :
					std::cout << "accessDenied";
					status=false;
					break;
				default:
					std::cerr << "Invalid AuthorizationCheckResult" << std::endl;
					status=false;
					break;
			}
			std::cout << "\tStatus Text:\t" << subStatus.getStatusText().c_str() << std::endl;
		}
		catch( rfa::dacs::AuthorizationException& ae )
		{
			std::cerr << "AuthorizationException" << std::endl;
			std::cerr << "StatusText : " << ae.getStatus()->getStatusText().c_str() << std::endl;
			status=false;
		}
	}else
	{
		std::cerr << "checkSubSBE() failed; Not logged in." << std::endl;
		status=false;
	}
	return status;
   
}
bool DacsClient::checkSubCBE( const rfa::common::RFA_String& serviceName, 
		const rfa::common::RFA_String& itemName, const rfa::common::RFA_Vector<unsigned long>& peList, rfa::dacs::AuthorizationLock::Operator op, bool reperm)
{
	bool status=false;
	rfa::dacs::AuthorizationLock* authLock;
	rfa::dacs::AuthorizationLockData authLockData;
	rfa::dacs::AuthorizationLockStatus lockStatus;
	if( _loginStatus == LoginSucceeded )
	{
		try{
			// Set an appropriate usage logging level for a check subscription.
			rfa::dacs::AuthorizationRequest::PerformUsage usage;
			if( reperm )
				usage = rfa::dacs::AuthorizationRequest::OnlyDenialPerformUsageLogging;
			else
				usage = rfa::dacs::AuthorizationRequest::AlwaysPerformUsageLogging;

			// Will contain status information returned from SBE check
			rfa::dacs::AuthorizationCheckStatus subStatus;

			std::cout << std::endl << "Performing CBE check with the following credentials:\n";
			std::cout << "\tUser:\t\t" << _spId.getName().c_str() << std::endl;
			std::cout << "\tPosition:\t" << _spId.getPosition().c_str() << std::endl;
			std::cout << "\tApplication Id:\t" << _spId.getAppName().c_str() << "\n\n";


		
			authLock = _authAgent->createAuthorizationLock(serviceName, op, peList, lockStatus);

			if(authLock == NULL)
			{
				std::cerr<<" AuthorizationAgent::createAuthorizationLock returns error:  "<<lockStatus.getStatusText().c_str()<<std::endl;
				return false;
			}

			if (authLock->getLock(authLockData, lockStatus) == rfa::dacs::AuthorizationLockData::LOCK_FAILURE)
			{
				std::cerr<<" AuthorizationAgentLock::getLock returns error:  "<<lockStatus.getStatusText().c_str()<<std::endl;
				delete authLock;
				return false;
			}

			// Check User is permissioned for service/item
			rfa::dacs::AuthorizationAgent::AuthorizationCheckResult result =
					_authAgent->checkSubscription(
						*_phLogin, usage, subStatus, serviceName, itemName,
						authLockData.size(), authLockData.c_lockData());

			status=(result==rfa::dacs::AuthorizationAgent::accessAllowed);
			switch( result  )
			{
			case rfa::dacs::AuthorizationAgent::accessAllowed :
					std::cout<<"accessAllowed DACS Lock "<<std::endl;					
					break;
			case rfa::dacs::AuthorizationAgent::accessDenied :
					//If the result is denied make sure we report the StatusText
					// this will indicate what service/sub-service is required.
					std::cout<<"accessDenied to DACS Lock "<<std::endl;					
					std::cout<<" statusText: "<<subStatus.getStatusText().c_str()<<std::endl;
					break;
				default:
					std::cerr<<"Invalid AuthorizationCheckResult"<<std::endl;
			}
		}
		catch( rfa::dacs::AuthorizationException& ae )
		{
			std::cerr << "AuthorizationException" << std::endl;
			std::cerr << "StatusText : " << ae.getStatus()->getStatusText().c_str() << std::endl;
			status=false;
		}
	}
	else
	{
		std::cerr << "checkSubCBE() failed; Not logged in." << std::endl;
		status=false;
	}
	delete authLock;
	return status;
}

bool DacsClient::checkSubCBE( const rfa::common::RFA_String& serviceName, 
		const rfa::common::RFA_String& itemName, const rfa::common::Buffer* permData, bool reperm)
{
	bool status=false;

	if(permData != 0)
	{
		if( _loginStatus == LoginSucceeded )
		{
			try{
			// Set an appropriate usage logging level for a check subscription.
				rfa::dacs::AuthorizationRequest::PerformUsage usage;
				if( reperm )
					usage = rfa::dacs::AuthorizationRequest::OnlyDenialPerformUsageLogging;
				else
					usage = rfa::dacs::AuthorizationRequest::AlwaysPerformUsageLogging;

				// Will contain status information returned from SBE check
				rfa::dacs::AuthorizationCheckStatus subStatus;

				decodeDacsLock(permData->c_buf(), permData->size());

				rfa::dacs::AuthorizationAgent::AuthorizationCheckResult result =
						_authAgent->checkSubscription(
							*_phLogin, usage, subStatus, serviceName, itemName,
							permData->size(), permData->c_buf());

				status=(result==rfa::dacs::AuthorizationAgent::accessAllowed);
				switch( result  )
				{
				case rfa::dacs::AuthorizationAgent::accessAllowed :
						std::cout<<"accessAllowed DACS Lock "<<std::endl;					
						break;
				case rfa::dacs::AuthorizationAgent::accessDenied :
						//If the result is denied make sure we report the StatusText
						// this will indicate what service/sub-service is required.
						std::cout<<"accessDenied to DACS Lock "<<std::endl;					
						std::cout<<" statusText: "<<subStatus.getStatusText().c_str()<<std::endl;
						break;
					default:
						std::cerr<<"Invalid AuthorizationCheckResult"<<std::endl;
				}

			}
			catch( rfa::dacs::AuthorizationException& ae )
			{
				std::cerr << "AuthorizationException" << std::endl;
				std::cerr << "StatusText : " << ae.getStatus()->getStatusText().c_str() << std::endl;
				status=false;
			}


		}else
		{
			std::cerr << "checkSubCBE() failed; Not logged in." << std::endl;
			status=false;
		}
	}else
	{
		std::cout << "No permission Data." << std::endl;
		status = true;
	}

	return status;
}

void DacsClient::decodeDacsLock( const unsigned char* lock, int len)
{
		rfa::dacs::AuthorizationLockData lockData((unsigned char*) lock, len);
		// Note that a cast is required if we use Buffer directly because 
		// there is a const mis-match between the Buffer().c_buf() method
		// which returns a const unsigned char* and
		// AuthorizationLockData c'tor which expects an unsigned char*
			//Create a RFA_Vector to store AuthorizationLock objects.
		rfa::common::RFA_Vector<rfa::dacs::AuthorizationLock> vLocks;
		//  Create an AuthorizationLockStatus object to contain the result of
		// extracting the AuthorizationLock objects from the
		// AuthorizationLockData object.	
		rfa::dacs::AuthorizationLockStatus lockStatus;
		//Use AuthorizationLockData::enumLockdata() method to extract the 
		// AuthorizationLock objects from AuthorizationLockData...
		rfa::dacs::AuthorizationLockData::LockResult lockResult = 
			lockData.enumLockdata( vLocks, lockStatus);
		if( lockResult == rfa::dacs::AuthorizationLockData::LOCK_FAILURE )
			printf("Failed to decode DACS Lock: %s",lockStatus.getStatusText().c_str());
		else
		{ 
			//If enumLockdata() succeeded then vLocks contains at least one 
			// AuthorizationLock, examine each one in turn...
			for ( unsigned int iLocks=0; iLocks != vLocks.size(); ++iLocks )
			{
				rfa::dacs::AuthorizationLock& authLock=vLocks[iLocks];
				printf("ServiceID: %d  PE List : ",authLock.getServiceID() );
				// Each AuthorizationLock object will contain at least one PE 
				// and an operator which indicates if entitlement is required to
				// all the PEs in the AuthorizationLock or any one of them.
				// Extract the PEs from the AuthorizationLock into a RFA_Vector...
				const rfa::common::RFA_Vector<unsigned long>& vPEs = authLock.getPeList();
				//Examine each PE value in vPEs in turn...
				unsigned long pe=0;
				for( unsigned int ipe=0; ipe < vPEs.size(); ++ipe)
				{
					pe=vPEs[ipe];
					printf(" %d",pe);
				}
				const char *lockOp=(authLock.getOperator()==rfa::dacs::AuthorizationLock::OR)?"OR":
					((authLock.getOperator()==rfa::dacs::AuthorizationLock::AND)?"AND":"BAD VALUE");
				printf(" Operator: %s\n",lockOp);
			}
		}
}
//void decodeDacsLock( const rfa::common::Buffer& buf )
//	{
//		unsigned char* pLock = new unsigned char[buf.size()];
//		memcpy(pLock,buf.c_buf(),buf.size());
//		//The buffer delivered with the PermissionData message type is a 
//		// DACS lock, so we create an AuthorizationLockData object from it...
//		// 
//		using rfa::dacs::AuthorizationLockData;
//		AuthorizationLockData lockData( pLock, buf.size());
//		// Note that a cast is required if we use Buffer directly because 
//		// there is a const mis-match between the Buffer().c_buf() method
//		// which returns a const unsigned char* and
//		// AuthorizationLockData c'tor which expects an unsigned char*
//			//Create a RFA_Vector to store AuthorizationLock objects.
//		using rfa::dacs::AuthorizationLock;
//		using rfa::common::RFA_Vector;
//		RFA_Vector<AuthorizationLock> vLocks;
//		//  Create an AuthorizationLockStatus object to contain the result of
//		// extracting the AuthorizationLock objects from the
//		// AuthorizationLockData object.
//		using rfa::dacs::AuthorizationLockStatus;
//		AuthorizationLockStatus lockStatus;
//		//Use AuthorizationLockData::enumLockdata() method to extract the 
//		// AuthorizationLock objects from AuthorizationLockData...
//		AuthorizationLockData::LockResult lockResult = 
//			lockData.enumLockdata( vLocks, lockStatus);
//		if( lockResult == AuthorizationLockData::LOCK_FAILURE )
//			printf("Failed to decode DACS Lock: %s",lockStatus.getStatusText().c_str());
//		else
//		{ 
//			//If enumLockdata() succeeded then vLocks contains at least one 
//			// AuthorizationLock, examine each one in turn...
//			for ( unsigned int iLocks=0; iLocks != vLocks.size(); ++iLocks )
//			{
//				AuthorizationLock& authLock=vLocks[iLocks];
//				printf("ServiceID: %d  PE List : ",authLock.getServiceID() );
//				// Each AuthorizationLock object will contain at least one PE 
//				// and an operator which indicates if entitlement is required to
//				// all the PEs in the AuthorizationLock or any one of them.
//				// Extract the PEs from the AuthorizationLock into a RFA_Vector...
//				const RFA_Vector<unsigned long>& vPEs = authLock.getPeList();
//				//Examine each PE value in vPEs in turn...
//				unsigned long pe=0;
//				for( unsigned int ipe=0; ipe < vPEs.size(); ++ipe)
//				{
//					pe=vPEs[ipe];
//					printf(" %d",pe);
//				}
//				const char *lockOp=(authLock.getOperator()==AuthorizationLock::OR)?"OR":
//					((authLock.getOperator()==AuthorizationLock::AND)?"AND":"BAD VALUE");
//				printf(" Operator: %s\n",lockOp);
//			}
//		}
//	}



