/*
	basicConsumer - Tutorial 4 - Retrieving level 1 content and decoding the results
	The goal of this tutorial is to extend Tutorial 3 to request and retrieve level 1 (MarketPrice) data from the OMM-based data
	Provider.  To support the level1 content, we offload the decoding of messages into separate files to allow
	for easy management and support for additional functionality.

	The series includes the following:

		o Tutorial 1 - Creating a basic barebones UPA consumer application
		o Tutorial 2 - Establishing a connection to an OMM-based data Provider
		o Tutorial 3 - Establishing a session with an OMM-based data Provider
		o Tutorial 4 - Retrieving level 1 content and decoding the results
		o Tutorial 5 - Retrieving level 2 content and decoding the results
 */

/* System headers */
#include <stdlib.h>
#include <iostream>

/* Application headers */
#include "basicConsumer.h"
#include "basicDirectoryHandler.h"
#include "basicDictionaryHandler.h"
#include "basicMsgHandler.h"

/* UPA headers */
#include "rtr/rsslReactor.h"

rfa::common::RFA_String appUserName;
rfa::common::RFA_String itemUserName;
rfa::common::RFA_String hostName;
rfa::common::RFA_String port;
rfa::common::RFA_String serviceName;
rfa::common::RFA_String itemName;
rfa::common::RFA_String dacsConnectionStr;
rfa::common::RFA_String appId;
rfa::common::RFA_String position;

/* Name of our market data service, item and domain of our subscription */
#define SERVICE	"ELEKTRON_EDGE"
RsslUInt8 domainType = RSSL_DMT_MARKET_PRICE;

/* RDM objects used at startup */
RsslReactorConnectOptions connectionOptions;
RsslRDMLoginRequest loginRequest;				// Login Object
RsslRDMDirectoryRequest directoryRequest;		// Directory Object
BasicService service = { SERVICE, 0 };			// Service Object

/* RsslReactor components*/
fd_set readFds, exceptFds;
RsslReactor *pReactor = NULL;

/* Keep RsslReactorChannel */
RsslReactorChannel* pRsslReactorChannel = NULL;

/* Keep DacsClient instance */
DacsClient* pDacsClient = NULL; 
rfa::common::EventQueue* pEventQueue = NULL; // Event Queue used to be dispatched for Open DACS event

rfa::common::StandardPrincipalIdentity spi;
rfa::common::Buffer _dacsLock;
int isSubscribed = 0;

/* Local functions */
void init();
void mainEventLoop();
RsslReactorCallbackRet channelEventCallback(RsslReactor *pReactor, RsslReactorChannel *pReactorChannel, RsslReactorChannelEvent *pConnEvent);
RsslReactorCallbackRet loginMsgCallback(RsslReactor *pReactor, RsslReactorChannel *pChannel, RsslRDMLoginMsgEvent *pLoginMsgEvent);
RsslReactorCallbackRet directoryMsgCallback(RsslReactor *pReactor, RsslReactorChannel *pChannel, RsslRDMDirectoryMsgEvent *pDirectoryMsgEvent);
RsslReactorCallbackRet dictionaryMsgCallback(RsslReactor *pReactor, RsslReactorChannel *pReactorChannel, RsslRDMDictionaryMsgEvent *pDictionaryMsgEvent);
RsslReactorCallbackRet defaultMsgCallback(RsslReactor *pReactor, RsslReactorChannel *pChannel, RsslMsgEvent* pMsgEvent);
void closeConnection(RsslReactor *pReactor, RsslReactorChannel *pChannel);
void cleanUpAndExit(int code);
void exitApp(int code);

int getCmdParams(
	int argc, char* argv[],
	rfa::common::RFA_String &appUserName,
	rfa::common::RFA_String &itemUserName,
	rfa::common::RFA_String &hostName,
	rfa::common::RFA_String &port,
	rfa::common::RFA_String &itemName,
	rfa::common::RFA_String &serviceName,
	rfa::common::RFA_String &dacsConnectionStr,
	rfa::common::RFA_String &appId,
	rfa::common::RFA_String &position
	)
{
	int ret = 0;
	int required = 0;
	// defaults

	position = "1.0.0.0/net";
	appId = "256";
	port = "14002";
	dacsConnectionStr = "local";
	// check the parameters
	for (int i = 1; i < argc; i++)
	{
		// if the serviceName was specified
		if ((strcmp(argv[i], "-appUser") == 0) && ((i + 1) < argc))
		{
			appUserName = argv[++i];
			++required;
		}

		else if ((strcmp(argv[i], "-position") == 0) && ((i + 1) < argc))
		{
			position = argv[++i];
		}
		else if ((strcmp(argv[i], "-appid") == 0) && ((i + 1) < argc))
		{
			appId = argv[++i];
		}
		else if ((strcmp(argv[i], "-s") == 0) && ((i + 1) < argc))
		{
			serviceName = argv[++i];
			++required;
		}
		else if ((strcmp(argv[i], "-i") == 0) && ((i + 1) < argc))
		{
			itemName = argv[++i];
			++required;
		}
		else if ((strcmp(argv[i], "-itemUser") == 0) && ((i + 1) < argc))
		{
			itemUserName = argv[++i];
			++required;
		}
		else if ((strcmp(argv[i], "-d") == 0) && ((i + 1) < argc))
		{
			dacsConnectionStr = argv[++i];
			
		}
		else if ((strcmp(argv[i], "-h") == 0) && ((i + 1) < argc))
		{
			hostName = argv[++i];
			++required;
		}
		else if ((strcmp(argv[i], "-p") == 0) && ((i + 1) < argc))
		{
			port = argv[++i];
		
		}
	}
	// if we didn't read all the mandatory parameters
	if (required != 5)
	{
		ret = 1;
		std::cerr << "Usage: Consumer -s <serviceName> -i <itemName> -h <hostname or Ip> -appUser <username for app> -itemUser <username for item> [-d <dacs server:port>] [-position <position>] [-appid <application ID>] [-p <TCP port>]" << std::endl;
		std::cerr << "e.g.   Consumer -s hEED -i TRI.N -h ads1 -appUser all -itemUser user01" << std::endl;
	}

	return ret;
}

/*** MAIN ***/
int main( int argc, char* argv[] )
{
	int ret = 0;

	ret = getCmdParams(
		argc, argv,
		appUserName,
		itemUserName,
		hostName,
		port,
		itemName,
		serviceName,
		dacsConnectionStr,
		appId,
		position
		);

	if (!ret){

		std::cout << "Username for Application = " << appUserName.c_str() << std::endl;
		std::cout << "Username for Item = " << itemUserName.c_str() << std::endl;
		std::cout << "Host name = " << hostName.c_str() << std::endl;
		std::cout << "Port = " << port.c_str() << std::endl;
		std::cout << "Position = " << position.c_str() << std::endl;
		std::cout << "Application ID = " << appId.c_str() << std::endl;
		std::cout << "Service Name = " << serviceName.c_str() << std::endl;
		std::cout << "Item Name = " << itemName.c_str() << std::endl;
		std::cout << "DACS Connection String = " << dacsConnectionStr.c_str() << std::endl;

		// Initialization
		init();

		// Main Event Loop
		mainEventLoop();

		// Exit
		cleanUpAndExit(0);
	}
}

/* Initialize RSSL and setup the main communication protocol details as outlined in the 3 steps within the introduction */
void init()
{
	RsslError error;

	RsslReactorOMMConsumerRole consumerRole;		// Channel Consumer role - client (consumer) connecting into a server (provider)
	RsslCreateReactorOptions reactorOpts;
	RsslErrorInfo rsslErrorInfo;

	/* Initialize RSSL. The locking mode RSSL_LOCK_GLOBAL_AND_CHANNEL is required to use the RsslReactor. */
	/* RsslReactor internally utilizes multiple threads */
	if (rsslInitialize(RSSL_LOCK_GLOBAL_AND_CHANNEL, &error) != RSSL_RET_SUCCESS)
	{
		printf("rsslInitialize(): failed <%s>\n", error.text);
		exit(RSSL_RET_FAILURE);
	}

	//Convert User name to RsslBuffer
	RsslBuffer user;
	user.data = const_cast<char*>(appUserName.c_str());
	user.length = appUserName.length();
	loginRequest.userName = user;


	strcpy(service.serviceName,serviceName.c_str());

	/* After this consumer successfully connects into a server (TREP), we must log in.*/
	/* Initialize a default login request (Use 1 as the Login Stream ID). */
	if (rsslInitDefaultRDMLoginRequest(&loginRequest, 1) != RSSL_RET_SUCCESS)
	{
		printf("rsslInitDefaultRDMLoginRequest() failed\n");
		cleanUpAndExit(-1);
	}

	/* Initialize the default directory request (Use 2 as the Directory Stream Id) */
	if (rsslInitDefaultRDMDirectoryRequest(&directoryRequest, 2) != RSSL_RET_SUCCESS)
	{
		printf("rsslInitDefaultRDMDirectoryRequest() failed\n");
		cleanUpAndExit(-1);
	}

	rsslClearOMMConsumerRole(&consumerRole);

	/* Set the messages to send when the channel is up */
	consumerRole.pLoginRequest = &loginRequest;
	consumerRole.pDirectoryRequest = &directoryRequest;
	
	/**************************************************************************
	 * Setup callback functions for our connection channel - consumer role 
	 **************************************************************************/

	/* Login callback */
	consumerRole.loginMsgCallback = loginMsgCallback;

	/* Directory callback */
	consumerRole.directoryMsgCallback = directoryMsgCallback;

	/* Connection/Channel callback */
	consumerRole.base.channelEventCallback = channelEventCallback;

	/* Market Data Message callback */
	consumerRole.base.defaultMsgCallback = defaultMsgCallback;

	/* Load Dictionaries - local or server */
	consumerRole.dictionaryMsgCallback = loadLocalDictionary() ? NULL : dictionaryMsgCallback;
	consumerRole.dictionaryDownloadMode = (consumerRole.dictionaryMsgCallback ? RSSL_RC_DICTIONARY_DOWNLOAD_FIRST_AVAILABLE :
																				RSSL_RC_DICTIONARY_DOWNLOAD_NONE);

	/* Create our rsslReactor used to manage the UPA channel */
	rsslClearCreateReactorOptions(&reactorOpts);
	if (!(pReactor = rsslCreateReactor(&reactorOpts, &rsslErrorInfo)))
	{
		printf("Error: %s", rsslErrorInfo.rsslError.text);
		exit(-1);
	}

	// Prepare our connection properties
	rsslClearReactorConnectOptions(&connectionOptions);

	connectionOptions.rsslConnectOptions.connectionInfo.unified.address = const_cast<char*>(hostName.c_str());
	connectionOptions.rsslConnectOptions.connectionInfo.unified.serviceName = const_cast<char*>(port.c_str());
	connectionOptions.rsslConnectOptions.guaranteedOutputBuffers = 500;
	connectionOptions.rsslConnectOptions.majorVersion = RSSL_RWF_MAJOR_VERSION;
	connectionOptions.rsslConnectOptions.minorVersion = RSSL_RWF_MINOR_VERSION;
	connectionOptions.reconnectAttemptLimit = -1;	// Keep trying to reconnect upon failure
	connectionOptions.reconnectMinDelay = 5000;		// Reconnect interval (milliseconds)
	connectionOptions.reconnectMaxDelay = 10000;
	
	/* Reactor fd monitoring */
	FD_ZERO(&readFds);
	FD_ZERO(&exceptFds);

	/* Set the reactor's event file descriptor on our descriptor set. This, along with the file descriptors
	 * of RsslReactorChannels, will notify us when we should call rsslReactorDispatch(). */
	FD_SET(pReactor->eventFd, &readFds);

	/* Initialize connection */
	if (rsslReactorConnect(pReactor, &connectionOptions, (RsslReactorChannelRole*)&consumerRole, &rsslErrorInfo) != RSSL_RET_SUCCESS)
	{
		printf("Error rsslReactorConnect(): %s\n", rsslErrorInfo.rsslError.text);
		exit(-1);
	}

	pEventQueue = rfa::common::EventQueue::create(rfa::common::RFA_String("EventQueue"));
	
	spi.setName(itemUserName);
	spi.setPosition(position);
	spi.setAppName(appId);

	pDacsClient= new DacsClient(*pEventQueue, spi, dacsConnectionStr);
}

void mainEventLoop()
{
	RsslReactorDispatchOptions dispatchOpts;
	RsslErrorInfo rsslErrorInfo;
	int ret;

	rsslClearReactorDispatchOptions(&dispatchOpts);

	/* Main loop - select() to trigger event notification then dispatch (rsslReactorDispatch()) */
	do
	{
		fd_set useReadFds = readFds, useExceptFds = exceptFds;

		/* detect channel notifications */
		ret = select(FD_SETSIZE, &useReadFds, NULL, &useExceptFds, NULL);

		if (ret < 0)
		{
#ifdef _WIN32
                        if (WSAGetLastError() == WSAEINTR)
                                continue;

                        printf("Error: select: %d\n", WSAGetLastError());
#else
                        if (errno == EINTR)
                                continue;

                        perror("select");
#endif
                        cleanUpAndExit(-1);
		}

		/* Call rsslReactorDispatch().  This will handle any events that have occurred on our rssl channel and
		 * dispatched via the callbacks specified for our consumerRole as referenced within init().
		   A return value of RSSL_RET_SUCCESS indicates there may be more to process */
		while ((ret = rsslReactorDispatch(pReactor, &dispatchOpts, &rsslErrorInfo)) > RSSL_RET_SUCCESS)
			;

		if (ret < RSSL_RET_SUCCESS)
			printf("rsslReactorDispatch() failed: %s\n", rsslErrorInfo.rsslError.text);

		/* Dispatch DACS events to callback methods. */
		if (pEventQueue)
		{
			long dispatchReturn = pEventQueue->dispatch(10);
		}

	} while (ret >= RSSL_RET_SUCCESS);
}

/*
* Processes events about the state of an RsslReactorChannel.
*/
RsslReactorCallbackRet channelEventCallback(RsslReactor *pReactor, RsslReactorChannel *pReactorChannel, RsslReactorChannelEvent *pConnEvent)
{
	switch (pConnEvent->channelEventType)
	{
		case RSSL_RC_CET_CHANNEL_UP:
		{
			/* A channel that we have requested via rsslReactorConnect() has come up.  We must now
			 * associated the new channel descriptor within our select() watchlist. This will drive 
			 * the process of detecting message events, i.e. Login, Directory, Dictionary etc. */
			printf("Connection up! Channel fd=%d\n\n", pReactorChannel->socketId);

			/* Set file descriptor. */
			FD_SET(pReactorChannel->socketId, &readFds);
			FD_SET(pReactorChannel->socketId, &exceptFds);

			/* Note: Refer to the complete rsslConsumer example to set channel properites appropriate for your environment */
			break;
		}
		case RSSL_RC_CET_CHANNEL_READY:
			pRsslReactorChannel = pReactorChannel;

			printf("\nChannel is ready.\n");
		
			if (pDacsClient->getLoginStatus() == DacsClient::LoginSucceeded)
			{
				printf("\nAlready logged in to DACS system, Subscribing to %s\n", itemName.c_str());
				processDacsLogin(pDacsClient);
			} else 
			{
				printf("\nInitialize Open DACS components\n"); 
				if (pDacsClient->init() != 0) //DACSAUTH_SUCCESS
				{
					printf("DACS initialization failed\n");
					cleanUpAndExit(-1);
				}
			}
			break;

		case RSSL_RC_CET_FD_CHANGE:
		{
			/* The file descriptor representing the RsslReactorChannel has been changed.
			* Update our file descriptor sets. */
			printf("Fd change: %d to %d\n", pReactorChannel->oldSocketId, pReactorChannel->socketId);
			FD_CLR(pReactorChannel->oldSocketId, &readFds);
			FD_CLR(pReactorChannel->oldSocketId, &exceptFds);
			FD_SET(pReactorChannel->socketId, &readFds);
			FD_SET(pReactorChannel->socketId, &exceptFds);
			break;
		}
		case RSSL_RC_CET_CHANNEL_DOWN:
		{
			/* The channel has failed and has gone down.  Print the error, close the channel, and reconnect later. */
			printf("Connection down: Channel fd=%d.\n", pReactorChannel->socketId);

			if (pConnEvent->pError)
				printf("	Error text: %s\n\n", pConnEvent->pError->rsslError.text);

			closeConnection(pReactor, pReactorChannel);
			break;
		}
		case RSSL_RC_CET_CHANNEL_DOWN_RECONNECTING:
		{
			printf("Connection down, reconnecting.  Channel fd=%d\n", pReactorChannel->socketId);

			if (pReactorChannel->socketId != REACTOR_INVALID_SOCKET)
			{
				FD_CLR(pReactorChannel->socketId, &readFds);
				FD_CLR(pReactorChannel->socketId, &exceptFds);
			}
			break;
		}
		case RSSL_RC_CET_WARNING:
		{
			/* We have received a warning event for this channel. Print the information and continue. */
			printf("Received warning for Channel fd=%d.\n", pReactorChannel->socketId);
			printf("	Error text: %s\n\n", pConnEvent->pError->rsslError.text);
			break;
		}
		default:
		{
			printf("Unknown connection event!\n");
			cleanUpAndExit(-1);
		}
	}

	return RSSL_RC_CRET_SUCCESS;
}

/*
* Processes the information contained in Login responses.
* Copies the refresh so we can use it to know what features are
* supported(for example, whether posting is supported
* by the provider).
*/
RsslReactorCallbackRet loginMsgCallback(RsslReactor *pReactor, RsslReactorChannel *pChannel, RsslRDMLoginMsgEvent *pLoginMsgEvent)
{
	RsslBuffer tempBuffer = { 1024, (char*)alloca(1024) };
	RsslRDMLoginMsg *pLoginMsg = pLoginMsgEvent->pRDMLoginMsg;

	// Verify login event
	if (!pLoginMsg)
	{
		RsslErrorInfo *pError = pLoginMsgEvent->baseMsgEvent.pErrorInfo;
		printf("loginMsgCallback: %s(%s)\n", pError->rsslError.text, pError->errorLocation);
		closeConnection(pReactor, pChannel);
		return RSSL_RC_CRET_SUCCESS;
	}

	// Determine login type
	switch (pLoginMsg->rdmMsgBase.rdmMsgType)
	{
	case RDM_LG_MT_REFRESH:
	{
		RsslRDMLoginRefresh *pLoginRefresh = &pLoginMsg->refresh;

		/* get state information */
		rsslStateToString(&tempBuffer, &pLoginRefresh->state);
		printf("\nReceived Login response: %.*s\n\n", tempBuffer.length, tempBuffer.data);

		break;
	}
	case RDM_LG_MT_STATUS:
		printf("\nReceived Login StatusMsg\n");
		if (pLoginMsg->status.flags & RDM_LG_STF_HAS_STATE)
		{
			/* get state information */
			rsslStateToString(&tempBuffer, &pLoginMsg->status.state);
			printf("	%.*s\n\n", tempBuffer.length, tempBuffer.data);
		}
		break;

	default:
		printf("\nReceived Unhandled Login Msg Class: %d\n", pLoginMsg->rdmMsgBase.rdmMsgType);
		break;
	}

	return RSSL_RC_CRET_SUCCESS;
}

/*
* Processes information contained in Directory responses.
* Searches the refresh for the service the application
* wants to use for this connection and stores the information
* so we can request items from that service.
*/
RsslReactorCallbackRet directoryMsgCallback(RsslReactor *pReactor, RsslReactorChannel *pChannel, 
											RsslRDMDirectoryMsgEvent *pDirectoryMsgEvent)
{
	RsslRDMDirectoryMsg *pDirectoryMsg = pDirectoryMsgEvent->pRDMDirectoryMsg;

	if (!pDirectoryMsg)
	{
		RsslErrorInfo *pError = pDirectoryMsgEvent->baseMsgEvent.pErrorInfo;
		printf("processDirectoryResponse: %s(%s)\n", pError->rsslError.text, pError->errorLocation);
		closeConnection(pReactor, pChannel);
		return RSSL_RC_CRET_FAILURE;
	}

	/* Process the directory response */
	return(processDirectoryResponse(pDirectoryMsgEvent, &service));
}

/*
* Processes information contained in Dictionary responses.
* Takes the payload of the messages and caches them
* using the RSSL utilities for decoding dictionary info.
*/
RsslReactorCallbackRet dictionaryMsgCallback(RsslReactor *pReactor, RsslReactorChannel *pReactorChannel, 
											 RsslRDMDictionaryMsgEvent *pDictionaryMsgEvent)
{
	char	errTxt[256];
	RsslBuffer errorText = { 255, (char*)errTxt };
	RsslReactorCallbackRet ret = RSSL_RC_CRET_SUCCESS;

	RsslRDMDictionaryMsg *pDictionaryMsg = pDictionaryMsgEvent->pRDMDictionaryMsg;

	if (!pDictionaryMsg)
	{
		RsslErrorInfo *pError = pDictionaryMsgEvent->baseMsgEvent.pErrorInfo;
		printf("dictionaryResponseCallback: %s(%s)\n", pError->rsslError.text, pError->errorLocation);
		ret = RSSL_RC_CRET_FAILURE;
	}
	else
		ret = processDictionaryResponse(pReactor, pReactorChannel, pDictionaryMsgEvent);

	if (ret == RSSL_RC_CRET_FAILURE)
		closeConnection(pReactor, pReactorChannel);

	return(ret);
}

/*
* Processes all RSSL messages that aren't processed by any domain-specific callback functions.  Responses for
* items requested by the function are handled here.
*
* Returns: RSSL_RC_CRET_SUCCESS (success), RSSL_RC_CRET_FAILURE (otherwise)
*/
RsslReactorCallbackRet defaultMsgCallback(RsslReactor *pReactor, RsslReactorChannel *pChannel, RsslMsgEvent* pMsgEvent)
{
	RsslReactorCallbackRet ret = generalMsgCallback(pReactor, pChannel, pMsgEvent);

	if (ret == RSSL_RC_CRET_FAILURE)
	{
		/* Because this application is single - threaded, it is safe to call it inside callback functions. */
		closeConnection(pReactor, pChannel);
	}

	return(ret);
}

/*
* Closes a channel in the RsslReactor
*/
void closeConnection(RsslReactor *pReactor, RsslReactorChannel *pChannel)
{
	RsslErrorInfo rsslErrorInfo;

	if (pChannel->socketId != REACTOR_INVALID_SOCKET)
	{
		FD_CLR(pChannel->socketId, &readFds);
		FD_CLR(pChannel->socketId, &exceptFds);
	}

	if (rsslReactorCloseChannel(pReactor, pChannel, &rsslErrorInfo) != RSSL_RET_SUCCESS)
	{
		printf("rsslReactorCloseChannel() failed: %s\n", rsslErrorInfo.rsslError.text);
	}
}


/*
 * Cleans up and exits the application.
 */
void cleanUpAndExit(int code)
{
	pEventQueue->deactivate();
	pEventQueue->destroy();

	delete pDacsClient;
	
	RsslErrorInfo rsslErrorInfo;

	if (pReactor && rsslDestroyReactor(pReactor, &rsslErrorInfo) != RSSL_RET_SUCCESS)
	{
		printf("Error cleaning up reactor: %s\n", rsslErrorInfo.rsslError.text);
		exitApp(-1);
	}

	rsslUninitialize();

	exitApp(code);
}

void exitApp(int code)
{
	printf("Exiting.\n");
	exit(code);
}

//When the access is denied it will close an item and then exit
void processAccessDeny() {
	if (isSubscribed != 0){
		printf("::UnSubscribe()\n");
		sendItemClosed(pRsslReactorChannel, domainType);
		isSubscribed = 0;
	}

	cleanUpAndExit(-1);
}

int checkSubCBE(RsslBuffer dacsLock)
{
	rfa::common::Buffer tmpBuf;
	int result = 1;
	/* Convert permission data to RFA Buffer */
	tmpBuf.setFrom((unsigned char*)dacsLock.data, dacsLock.length);

	/* Verify if the permission data is changed */
	if (_dacsLock != tmpBuf)
	{
		_dacsLock = tmpBuf;
		result = pDacsClient->checkSubCBE(serviceName, itemName, &_dacsLock) ? 1 : 0;
	}
	return result;
}

void processDacsLogin(DacsClient* client)
{
	printf("\n::processDacsLoging()\n");
	bool result = false;

	printf("::Subscribe()\n");
	if(pDacsClient->checkSubSBE(serviceName, itemName))
	{
		isSubscribed = sendItemRequest(pReactor, pRsslReactorChannel, const_cast<char*>(itemName.c_str()), service.serviceId, domainType);
		printf("Send MMT_MARKET_PRICE Request\n");
	}else
	{
		printf("\n\n<-- SBE Checking Failed -->\n");
		cleanUpAndExit(-1);
	}

}

void processDacsLogout(DacsClient* client)
{
	printf("::processDacsLogout()\n");
	processAccessDeny();
}

void processReperm(DacsClient* client)
{
	printf("::processReperm()\n");
	
	if (pDacsClient->checkSubSBE(serviceName, itemName) == false){
		printf("Failed reperm subject base entitlement for service: %s item: %s\n", serviceName.c_str(), itemName.c_str());
		processAccessDeny();
		return;
	}

	if (pDacsClient->checkSubCBE(serviceName, itemName, &_dacsLock, true) == false){
		printf("Failed reperm content base entitlement for service: %s item: %s\n", serviceName.c_str(), itemName.c_str());
		processAccessDeny();
		return;
	}
}