/*
 * basicDictionaryHandler
 * Manage all Dictionary-related processing.  Dictionaries are utilized to decode RWF (Reuters Wire Formatted) messages received from the providers.
 * Dictionaries can be retrieved and loaded in 2 ways:
 *
 * 1. Local files
 *	  There are 2 files required to load and process.  The first file is a field definition file which describes the field names, types, sizes etc.
 *	  The second file is an enumeration file which is utilized by those fields which has an enum type.
 *
 * 2. Downloading from the server
 *	  If no local files are available, a second option of downloading the dictionary files from the connected server can be performed.  In this
 *	  case, the details to what is required to download is made available within the directory response performed at system initiation.
 *
 */

#include "basicDictionaryHandler.h"
#include "rtr/rsslRDMDictionaryMsg.h"

/* field dictionary file name */
static const char *fieldDictionaryFileName = "RDMFieldDictionary";

/* enum table file name */
static const char *enumTableFileName = "enumtype.def";

/* Data Dictionary */
RsslDataDictionary dictionary;

/*
 * Loads the field/enumType dictionaries from a file.
 * Returns: true (success), false (otherwise)
 */
int loadLocalDictionary()
{
	char errTxt[256];
	RsslBuffer errorText = {255, (char*)errTxt};
	RsslBool fieldDictionaryLoaded = RSSL_FALSE, enumTableLoaded = RSSL_FALSE;

	/* Load dictionary from file if possible. Otherwise we will try to download it. */
	rsslClearDataDictionary(&dictionary);
	if (rsslLoadFieldDictionary(fieldDictionaryFileName, &dictionary, &errorText) < 0)
		printf("\nUnable to load field dictionary.  Will attempt to download from provider.\n\tError Text: %s\n", errorText.data);
	else
		fieldDictionaryLoaded = RSSL_TRUE;
			
	if (rsslLoadEnumTypeDictionary(enumTableFileName, &dictionary, &errorText) < 0)
		printf("\nUnable to load enum type dictionary.  Will attempt to download from provider.\n\tError Text: %s\n", errorText.data);
	else
		enumTableLoaded = RSSL_TRUE;

	if (!fieldDictionaryLoaded || !enumTableLoaded)
	{
		/* Dictionary could not be loaded from files. We will need to get them from the provider. */
		rsslDeleteDataDictionary(&dictionary);
		rsslClearDataDictionary(&dictionary);
	}
	else
		printf("\nSuccessfully loaded local dictionaries\n\n");

	return(fieldDictionaryLoaded && enumTableLoaded);
}

/*
 * Returns the data dictionary.
 */
RsslDataDictionary* getDictionary()
{
	return( &dictionary );
}

/*
 * Processes information contained in Dictionary responses.
 * Takes the payload of the messages and caches them
 * using the RSSL utilities for decoding dictionary info.
 */
RsslReactorCallbackRet processDictionaryResponse(RsslReactor *pReactor, RsslReactorChannel *pReactorChannel, 
												 RsslRDMDictionaryMsgEvent *pDictionaryMsgEvent)
{
	char	errTxt[256];
	RsslBuffer errorText = {255, (char*)errTxt};
	char tempData[1024];
	RsslBuffer tempBuffer;
	RsslInt32	fieldStreamId = 0, enumStreamId = 0;
	RsslDecodeIterator dIter;

	RsslRDMDictionaryMsg *pDictionaryMsg = pDictionaryMsgEvent->pRDMDictionaryMsg;

	tempBuffer.data = tempData;
	tempBuffer.length = 1024;

	switch(pDictionaryMsg->rdmMsgBase.rdmMsgType)
	{
	case RDM_DC_MT_REFRESH:
	{
		RsslRDMDictionaryRefresh *pRefresh = &pDictionaryMsg->refresh;

		if (pRefresh->flags & RDM_DC_RFF_HAS_INFO)
		{
			/* The first part of a dictionary refresh should contain information about its type.
			 * Save this information and use it as subsequent parts arrive. */

			switch(pRefresh->type)
			{
				case RDM_DICTIONARY_FIELD_DEFINITIONS: 
					fieldStreamId = pRefresh->rdmMsgBase.streamId; 
					break;
				case RDM_DICTIONARY_ENUM_TABLES: 
					enumStreamId = pRefresh->rdmMsgBase.streamId; 
					break;
				default: 
					printf("Unknown dictionary type %llu from message on stream %d\n", pRefresh->type, pRefresh->rdmMsgBase.streamId);
					return RSSL_RC_CRET_FAILURE;
			}
		}

		/* decode dictionary response */
		rsslClearDecodeIterator(&dIter);
		rsslSetDecodeIteratorRWFVersion(&dIter, pReactorChannel->majorVersion, pReactorChannel->minorVersion);

		/* Set the iterator to the payload of the response and pass it to
		 * the appropriate dictionary decoder. */
		rsslSetDecodeIteratorBuffer(&dIter, &pRefresh->dataBody);

		printf("\nReceived Dictionary Response: %.*s\n", pRefresh->dictionaryName.length, pRefresh->dictionaryName.length ? pRefresh->dictionaryName.data : "");

		if (pRefresh->rdmMsgBase.streamId == fieldStreamId)
		{
    		if (rsslDecodeFieldDictionary(&dIter, &dictionary, RDM_DICTIONARY_VERBOSE, &errorText) == RSSL_RET_SUCCESS)
			{
				if (pRefresh->flags & RDM_DC_RFF_IS_COMPLETE)
					printf("Field Dictionary complete.\n");
			}
			else
    		{
    			printf("Decoding Dictionary failed: %.*s\n", errorText.length, errorText.data);
				return(RSSL_RC_CRET_FAILURE);
    		}
		}
		else if (pRefresh->rdmMsgBase.streamId == enumStreamId)
		{
    		if (rsslDecodeEnumTypeDictionary(&dIter, &dictionary, RDM_DICTIONARY_VERBOSE, &errorText) == RSSL_RET_SUCCESS)
			{
				if (pRefresh->flags & RDM_DC_RFF_IS_COMPLETE)
					printf("Enumerated Types Dictionary complete.\n");
			}
			else
    		{
    			printf("Decoding Dictionary failed: %.*s\n", errorText.length, errorText.data);
				return(RSSL_RC_CRET_FAILURE);
    		}
		}
		else
		{
			printf("Received unexpected dictionary message on stream %d\n", pDictionaryMsg->rdmMsgBase.streamId);
		}
		break;
	}

	case RDM_DC_MT_STATUS:
		printf("\nReceived StatusMsg for dictionary\n");
		break;

	default:
		printf("\nReceived Unhandled Dictionary Msg Type: %d\n", pDictionaryMsg->rdmMsgBase.rdmMsgType);
		break;
	}

	return RSSL_RC_CRET_SUCCESS;
}

