/*
 * basicDirectoryHandler
 * Manage all directory-related processing. The directory represents all the details relevant to the server (provider).
 * These details include:
 *
 *	o A list of the services (eg, ELEKTRON, IDN_RDF etc)
 *  o The specific details for each service such as:
 *		- the service name
 *		- the corresponding service ID
 *		- status of the service (UP/DOWN)
 *		- service capabilities (the type of data it provides, i.e. Market Price, Market By Order, etc)
 *		- dictionary details
 *
 * It is up to the user of this information how they wish to utilize these details.  For example, if you wish to
 * retrieve data, you are required to capture and specify the service ID when making future requests.  If you are 
 * interested in downloading the dictionary, the details are provided here.  
 * Note:	when using the ValueAdd libraries, they will automatically process the dictionary details and request for 
 *			the details on your behalf - if instructed.
 */

#include "basicDirectoryHandler.h"
#include "rtr/rsslRDMDirectoryMsg.h"

/*
 * Processes information contained in Directory responses.
 * Searches the refresh for the service the application
 * wants to use for this connection and stores the information
 * so we can request items from that service.
 */
RsslReactorCallbackRet processDirectoryResponse(RsslRDMDirectoryMsgEvent *pDirectoryMsgEvent, BasicService* service)
{
	char tempData[1024];
	RsslBuffer tempBuffer;
	RsslRDMService *pService;
	RsslRDMDirectoryMsg *pDirectoryMsg = pDirectoryMsgEvent->pRDMDirectoryMsg;
	RsslRDMDirectoryRefresh *pRefresh;

	tempBuffer.data = tempData;
	tempBuffer.length = 1024;

 	switch(pDirectoryMsg->rdmMsgBase.rdmMsgType)
	{
	case RDM_DR_MT_REFRESH:
	{
		RsslUInt32 i;

		pRefresh = &pDirectoryMsg->refresh;
		printf("\nReceived Source Directory Response: ");
		rsslStateToString(&tempBuffer, &pRefresh->state);
		printf("	%.*s\n", tempBuffer.length, tempBuffer.data);

		for (i = 0; i <  pRefresh->serviceCount; ++i)
		{
			RsslBuffer tmpServiceNameBuffer;

			pService = &pRefresh->serviceList[i];

			printf("   Received serviceName: %.*s", pService->info.serviceName.length, pService->info.serviceName.data);
			if ( pService->flags & RDM_SVCF_HAS_STATE )
				printf(".  Service State: %s", pService->state.serviceState == 1 ? "Up" : "Down");
			printf("\n");

			/* check if name matches service name entered by user. 
			 * if it does, store the service ID and QoS. This will
			 * be used to make item requests later. */
			tmpServiceNameBuffer.data = service->serviceName;
			tmpServiceNameBuffer.length = (RsslUInt32)strlen(service->serviceName);
			if (rsslBufferIsEqual(&pService->info.serviceName, &tmpServiceNameBuffer))
			{
				service->serviceId = (RsslUInt16)pService->serviceId;
				printf("\tFound your service %s using serviceId: %d\n", service->serviceName, pService->serviceId);
			}
		}

		break;
	}

	case RDM_DR_MT_STATUS:
	{
		RsslRDMDirectoryStatus *pStatus = &pDirectoryMsg->status;
		printf("\nReceived Source Directory StatusMsg\n");
		if (pStatus->flags & RDM_DR_STF_HAS_STATE)
    	{
			rsslStateToString(&tempBuffer, &pStatus->state);
			printf("	%.*s\n\n", tempBuffer.length, tempBuffer.data);
    	}
		break;
	}

	default:
		printf("\nReceived Unhandled Source Directory Msg Type: %d\n", pDirectoryMsg->rdmMsgBase.rdmMsgType);
    	break;
	}

	return RSSL_RC_CRET_SUCCESS;
}
