/*
 * basicMsgHandler
 * Manage all message-related processing.  The routines defined within this module have been built with re-usability
 * in mind.  That is, the same routine to request for different data models (Market Price, Market By Price, Symbollist etc).
 * In addition, when responses and updates arrive, the routines have been organized based on the structure (list, map etc)
 * as opposed to DMT (Data Model Type).  This way, we can utilize a single processing routine to manage processing for
 * multiple DMT's.
 *
 * The data responses are contained with the raw RWF (Reuters Wire Format).  As such, we utilize the dictionary object
 * to decode these raw messages into messages we can process and read.
 */

/* Application includes */
#include "basicConsumer.h"
#include "basicMsgHandler.h"
#include "basicDictionaryHandler.h"
#include "rtr/rsslPayloadEntry.h"

#define MAX_MSG_SIZE 4096

// Private interface
RsslRet encodeItemRequest(RsslReactorChannel* pReactorChannel, RsslBuffer *msgBuf, char* item,
						  RsslUInt16 serviceId, RsslUInt8 domain);
RsslRet encodeItemClosed(RsslReactorChannel* pReactorChannel, RsslBuffer *msgBuf, 
					 	  RsslUInt8 domain, RsslInt32 streamId);

RsslRet sendMessage(RsslReactor *pReactor, RsslReactorChannel* chnl, RsslBuffer* msgBuf);
RsslInt32 getAvailableStreamID();
RsslRet processRDMResponse(RsslReactorChannel *pReactorChannel, RsslMsg *msg, RsslDecodeIterator* dIter);
RsslRet decodeFieldList(RsslDataDictionary *dictionary, RsslDecodeIterator *dIter);
RsslRet decodeFieldEntry(RsslDataDictionary *dictionary, RsslFieldEntry* fEntry, RsslDecodeIterator* dIter);

/* Request Streamd ID */
RsslInt32 availableStreamId = 100;
RsslInt32 currentStreamId = 0;
RsslReactor * _pReactor = 0;

/* sendItemClosed
 * Generic item request routine 
 */
RsslRet sendItemClosed(RsslReactorChannel* pReactorChannel, 
						RsslUInt8 domain)
{
	RsslErrorInfo rsslErrorInfo;
	RsslBuffer* msgBuf = rsslReactorGetBuffer(pReactorChannel, MAX_MSG_SIZE, RSSL_FALSE, &rsslErrorInfo);

	if (msgBuf != NULL)
	{
		/* encode item closed */
		if (encodeItemClosed(pReactorChannel, msgBuf, domain, currentStreamId) != RSSL_RET_SUCCESS)
		{
			rsslReactorReleaseBuffer(pReactorChannel, msgBuf, &rsslErrorInfo);
			printf("\nencodeItemClosed() failed\n");
			return RSSL_RET_FAILURE;
		}

		/* send item closed */
		if (sendMessage(_pReactor, pReactorChannel, msgBuf) != RSSL_RET_SUCCESS)
			return RSSL_RET_FAILURE;
	}
	else
	{
		printf("rsslReactorGetBuffer(): Failed <%s>\n", rsslErrorInfo.rsslError.text);
		return RSSL_RET_FAILURE;
	}

	return RSSL_RET_SUCCESS;
}

/* sendItemRequest
 * Generic item request routine 
 */
RsslRet sendItemRequest(RsslReactor *pReactor, RsslReactorChannel* pReactorChannel, 
						char* itemName, RsslUInt16 serviceId, RsslUInt8 domain)
{
	RsslErrorInfo rsslErrorInfo;
	RsslBuffer* msgBuf = rsslReactorGetBuffer(pReactorChannel, MAX_MSG_SIZE, RSSL_FALSE, &rsslErrorInfo);

	_pReactor = pReactor;

	if (msgBuf != NULL)
	{
		/* encode item request */
		if (encodeItemRequest(pReactorChannel, msgBuf, itemName, serviceId, domain) != RSSL_RET_SUCCESS)
		{
			rsslReactorReleaseBuffer(pReactorChannel, msgBuf, &rsslErrorInfo);
			printf("\nencodeItemRequest() failed\n");
			return RSSL_RET_FAILURE;
		}

		/* send item request */
		if (sendMessage(pReactor, pReactorChannel, msgBuf) != RSSL_RET_SUCCESS)
			return RSSL_RET_FAILURE;
	}
	else
	{
		printf("rsslReactorGetBuffer(): Failed <%s>\n", rsslErrorInfo.rsslError.text);
		return RSSL_RET_FAILURE;
	}

	return RSSL_RET_SUCCESS;
}

RsslRet encodeItemClosed(RsslReactorChannel* pReactorChannel, RsslBuffer *msgBuf, 
					 	  RsslUInt8 domain, RsslInt32 streamId)
{
	RsslRet ret = 0;
	RsslCloseMsg msg = RSSL_INIT_CLOSE_MSG;
	RsslEncodeIterator encodeIter;

	/* set-up message */
	msg.msgBase.msgClass = RSSL_MC_CLOSE;
	msg.msgBase.streamId = streamId;
	msg.msgBase.domainType = domain;
	msg.msgBase.containerType = RSSL_DT_NO_DATA;

	/* encode message */
	rsslClearEncodeIterator(&encodeIter);
	ret = rsslSetEncodeIteratorBuffer(&encodeIter, msgBuf);
	ret = rsslSetEncodeIteratorRWFVersion(&encodeIter, pReactorChannel->majorVersion, pReactorChannel->minorVersion);
	if ((ret = rsslEncodeMsgInit(&encodeIter, (RsslMsg*)&msg, 0)) < RSSL_RET_SUCCESS)
	{
		printf("rsslEncodeMsgInit() failed with return code: %d\n", ret);
		return ret;
	}

	if ((ret = rsslEncodeMsgComplete(&encodeIter, RSSL_TRUE)) < RSSL_RET_SUCCESS)
	{
		printf("rsslEncodeMsgComplete() failed with return code: %d\n", ret);
		return ret;
	}

	msgBuf->length = rsslGetEncodedBufferLength(&encodeIter);

	return RSSL_RET_SUCCESS;
}

/*
* Encodes the item request.  Returns success if
* encoding succeeds or failure if encoding fails.
* pReactorChannel - The channel to send an item request to
* msgBuf - The message buffer to encode the item request into
* pRequest - Item request structure
* isPrivateStream - Flag for private stream request
*
* This function is only used within the Market Price Handler
* and each handler has its own implementation, although much is similar
*/
RsslRet encodeItemRequest(RsslReactorChannel* pReactorChannel, RsslBuffer *msgBuf, 
					 	  char* item, RsslUInt16 serviceId, RsslUInt8 domain)
{
	RsslRet ret = 0;
	RsslRequestMsg msg = RSSL_INIT_REQUEST_MSG;
	RsslEncodeIterator encodeIter;

	currentStreamId = getAvailableStreamID();
	/* set-up message */
	msg.msgBase.msgClass = RSSL_MC_REQUEST;
	msg.msgBase.streamId = currentStreamId;
	msg.msgBase.domainType = domain;
	msg.msgBase.containerType = RSSL_DT_NO_DATA;
	msg.flags = RSSL_RQMF_HAS_QOS | RSSL_RQMF_STREAMING | RSSL_RQMF_HAS_PRIORITY;
	msg.priorityClass = 1;
	msg.priorityCount = 1;

	/* Specify the quality of service - take from the service details return from the directory */
	msg.qos.timeliness = RSSL_QOS_TIME_REALTIME;
	msg.qos.rate = RSSL_QOS_RATE_TICK_BY_TICK;

	/* specify msgKey members */
	msg.msgBase.msgKey.flags = RSSL_MKF_HAS_NAME_TYPE | RSSL_MKF_HAS_NAME | RSSL_MKF_HAS_SERVICE_ID;
	msg.msgBase.msgKey.nameType = RDM_INSTRUMENT_NAME_TYPE_RIC;
	msg.msgBase.msgKey.name.data = item;
	msg.msgBase.msgKey.name.length = strlen(item);
	msg.msgBase.msgKey.serviceId = serviceId;

	/* encode message */
	rsslClearEncodeIterator(&encodeIter);
	ret = rsslSetEncodeIteratorBuffer(&encodeIter, msgBuf);
	ret = rsslSetEncodeIteratorRWFVersion(&encodeIter, pReactorChannel->majorVersion, pReactorChannel->minorVersion);
	if ((ret = rsslEncodeMsgInit(&encodeIter, (RsslMsg*)&msg, 0)) < RSSL_RET_SUCCESS)
	{
		printf("rsslEncodeMsgInit() failed with return code: %d\n", ret);
		return ret;
	}

	if ((ret = rsslEncodeMsgComplete(&encodeIter, RSSL_TRUE)) < RSSL_RET_SUCCESS)
	{
		printf("rsslEncodeMsgComplete() failed with return code: %d\n", ret);
		return ret;
	}

	msgBuf->length = rsslGetEncodedBufferLength(&encodeIter);

	return RSSL_RET_SUCCESS;
}

/* getAvailableStreamdID 
 * When issuing a request to a provider, a unique ID should be used to avoid stream conflicts when processing 
 * responses and updates.
 *
 * Returns: the next available stream ID
 */
RsslInt32 getAvailableStreamID()
{
	/* Simple algorithm for uniqueness - however a more suitable ID table may be more appropriate */
	return(availableStreamId++);
}

/* Sends a message buffer to a channel
* pReactor - RsslReactor associated with the application
* chnl - The channel to send the message buffer to
* msgBuf - The msgBuf to be sent
*/
RsslRet sendMessage(RsslReactor *pReactor, RsslReactorChannel* chnl, RsslBuffer* msgBuf)
{
	RsslErrorInfo rsslErrorInfo;
	RsslRet	retval = 0;
	RsslUInt32 outBytes = 0;
	RsslUInt32 uncompOutBytes = 0;
	RsslUInt8 writeFlags = RSSL_WRITE_NO_FLAGS;
	RsslReactorSubmitOptions submitOpts;

	rsslClearReactorSubmitOptions(&submitOpts);

	/* send the request */
	if ((retval = rsslReactorSubmit(pReactor, chnl, msgBuf, &submitOpts, &rsslErrorInfo)) < RSSL_RET_SUCCESS)
	{
		while (retval == RSSL_RET_WRITE_CALL_AGAIN)
			retval = rsslReactorSubmit(pReactor, chnl, msgBuf, &submitOpts, &rsslErrorInfo);

		if (retval < RSSL_RET_SUCCESS)	/* Connection should be closed, return failure */
		{
			/* rsslWrite failed, release buffer */
			printf("rsslReactorSubmit() failed with return code %d - <%s>\n", retval, rsslErrorInfo.rsslError.text);
			if (retval = rsslReactorReleaseBuffer(chnl, msgBuf, &rsslErrorInfo) != RSSL_RET_SUCCESS)
				printf("rsslReactorReleaseBuffer() failed with return code %d - <%s>\n", retval, rsslErrorInfo.rsslError.text);

			return RSSL_RET_FAILURE;
		}
	}

	return RSSL_RET_SUCCESS;
}

/*
* Processes all RSSL messages that aren't processed by
* any domain-specific callback functions.  Responses for
* items requested by the function are handled here.
*/
RsslReactorCallbackRet generalMsgCallback(RsslReactor *pReactor, RsslReactorChannel *pChannel, RsslMsgEvent* pMsgEvent)
{
	RsslDecodeIterator dIter;
	RsslMsg *pMsg = pMsgEvent->pRsslMsg;

	if (!pMsg)
	{
		/* The message is not present because an error occurred while decoding it.  Print
		* the error and close the channel. If desired, the un-decoded message buffer
		* is available in pMsgEvent->pRsslMsgBuffer. */

		RsslErrorInfo *pError = pMsgEvent->pErrorInfo;
		printf("defaultMsgCallback: %s(%s)\n", pError->rsslError.text, pError->errorLocation);
		return RSSL_RC_CRET_FAILURE;
	}

	/* Clear the iterator and set it to the message payload. */
	rsslClearDecodeIterator(&dIter);
	rsslSetDecodeIteratorRWFVersion(&dIter, pChannel->majorVersion, pChannel->minorVersion);
	rsslSetDecodeIteratorBuffer(&dIter, &pMsg->msgBase.encDataBody);

	/* Process RDM msg */
	return(processRDMResponse(pChannel, pMsg, &dIter) == RSSL_RET_SUCCESS ? RSSL_RC_CRET_SUCCESS : RSSL_RC_CRET_FAILURE);
}

/*
* processRDMResponse
*
* The RDM response handler is a generic handler that is designed to decode RDM messages.  Depending on the
* RDM type, each response may contain 1 or more container types as part of the response package.  Depending on
* the type, an appropriate type decoder will be executed.
*
* pReactorChannel - Channel this message was received on.
* msg - The partially decoded message
* dIter - The decode iterator
*/
RsslRet processRDMResponse(RsslReactorChannel *pReactorChannel, RsslMsg *msg, RsslDecodeIterator* dIter)
{
	RsslRet ret = RSSL_RET_SUCCESS;
	char tempData[1024];
	RsslBuffer tempBuffer;
	RsslMsgKey* key = (RsslMsgKey *)rsslGetMsgKey(msg);

	tempBuffer.data = tempData;
	tempBuffer.length = sizeof(tempData);

	printf("\nReceived activity on channel: %d\n\n", pReactorChannel->socketId);

	switch (msg->msgBase.msgClass)
	{
	case RSSL_MC_REFRESH:
		if (msg->refreshMsg.flags & RSSL_STMF_HAS_PERM_DATA)
		{
			if (checkSubCBE(msg->refreshMsg.permData) == 0)
			{
				processAccessDeny();
				return -1;
			}
		}

		/* Echo instrument we requested - usually on REFRESH */
		if (key && (key->flags & RSSL_MKF_HAS_NAME))
			printf("Item: %.*s\n", key->name.length, key->name.data);

		rsslStateToString(&tempBuffer, &msg->refreshMsg.state);
		printf("%.*s\n", tempBuffer.length, tempBuffer.data);
		
		// Fall-through logic to handle the rest of the details (update)
	case RSSL_MC_UPDATE:
		printf("Domain: %s\n", rsslDomainTypeToString(msg->msgBase.domainType));

		switch (msg->msgBase.containerType)
		{
		case RSSL_DT_FIELD_LIST:
			ret = decodeFieldList(getDictionary(), dIter);
			break;

		default:
			printf("ContainType: %i not handled\n", msg->msgBase.containerType);
		}

		break;

	case RSSL_MC_STATUS:
		printf("\nReceive statusMsg for stream: %i\n", msg->statusMsg.msgBase.streamId);

		if (msg->statusMsg.flags & RSSL_STMF_HAS_STATE)
		{
			rsslStateToString(&tempBuffer, &msg->statusMsg.state);
			printf("	%.*s\n\n", tempBuffer.length, tempBuffer.data);
		}

		if (msg->statusMsg.flags & RSSL_STMF_HAS_PERM_DATA)
		{
			if (checkSubCBE(msg->statusMsg.permData) == 0)
			{
				processAccessDeny();
				return -1;
			}
		}

		break;

	default:
		printf("\nReceived Unhandled Item Msg Class: %d\n", msg->msgBase.msgClass);
		break;
	}

	return(ret);
}

/*
* decodeFieldList 
*/
RsslRet decodeFieldList(RsslDataDictionary *dictionary, RsslDecodeIterator *dIter)
{
	RsslRet ret;
	RsslFieldList fList = RSSL_INIT_FIELD_LIST;
	RsslFieldEntry fEntry = RSSL_INIT_FIELD_ENTRY;

	if ((ret = rsslDecodeFieldList(dIter, &fList, 0)) == RSSL_RET_SUCCESS)
	{
		/* decode each field entry in list */
		while ((ret = rsslDecodeFieldEntry(dIter, &fEntry)) != RSSL_RET_END_OF_CONTAINER)
		{
			if (ret == RSSL_RET_SUCCESS)
			{
				/* decode field entry info */
				if (decodeFieldEntry(dictionary, &fEntry, dIter) != RSSL_RET_SUCCESS)
				{
					printf("\ndecodeFieldEntry() failed\n");
					return RSSL_RET_FAILURE;
				}
			}
			else
			{
				printf("rsslDecodeFieldEntry() failed with return code: %d\n", ret);
				return RSSL_RET_FAILURE;
			}
		}
		if (ret == RSSL_RET_END_OF_CONTAINER)
			return RSSL_RET_SUCCESS;
	}
	else
	{
		printf("rsslDecodeFieldList() failed with return code: %d\n", ret);
		return RSSL_RET_FAILURE;
	}

	return ret;
}

/*
* decodeFieldEntry
*
* Decodes the field entry data and prints out the field entry data
* with help of the dictionary.  Returns success if decoding succeeds
* or failure if decoding fails.
* pReactorChannel - The channel the message was received on
* fEntry - The field entry data
* dIter - The decode iterator
*/
RsslRet decodeFieldEntry(RsslDataDictionary *dictionary, RsslFieldEntry* fEntry, RsslDecodeIterator *dIter)
{
	RsslRet ret = 0;
	RsslDataType dataType = RSSL_DT_UNKNOWN;
	RsslUInt64 fidUIntValue = 0;
	RsslInt64 fidIntValue = 0;
	RsslFloat tempFloat = 0;
	RsslDouble tempDouble = 0;
	RsslReal fidRealValue = RSSL_INIT_REAL;
	RsslEnum fidEnumValue;
	RsslFloat fidFloatValue = 0;
	RsslDouble fidDoubleValue = 0;
	RsslQos fidQosValue = RSSL_INIT_QOS;
	RsslDateTime fidDateTimeValue;
	RsslState fidStateValue;
	RsslBuffer fidBufferValue;
	RsslBuffer fidDateTimeBuf;
	RsslBuffer fidRealBuf;
	RsslBuffer fidStateBuf;
	RsslBuffer fidQosBuf;
	RsslDictionaryEntry* dictionaryEntry = NULL;

	/* get dictionary entry */
	if (!dictionary->isInitialized)
	{
		printf("\nDictionary not initialized - no output\n");
		return RSSL_RET_SUCCESS;
	}
	else
		dictionaryEntry = dictionary->entriesArray[fEntry->fieldId];

	/* return if no entry found */
	if (!dictionaryEntry)
	{
		printf("\tFid %d not found in dictionary\n", fEntry->fieldId);
		return RSSL_RET_SUCCESS;
	}

	/* print out fid name */
	printf("\t%-20s", dictionaryEntry->acronym.data);

	/* decode and print out fid value */
	dataType = dictionaryEntry->rwfType;
	switch (dataType)
	{
	case RSSL_DT_UINT:
		if ((ret = rsslDecodeUInt(dIter, &fidUIntValue)) == RSSL_RET_SUCCESS)
		{
			printf(""RTR_LLU"\n", fidUIntValue);
		}
		else if (ret != RSSL_RET_BLANK_DATA)
		{
			printf("rsslDecodeUInt() failed with return code: %d\n", ret);
			return ret;
		}
		break;
	case RSSL_DT_INT:
		if ((ret = rsslDecodeInt(dIter, &fidIntValue)) == RSSL_RET_SUCCESS)
		{
			printf(""RTR_LLD"\n", fidIntValue);
		}
		else if (ret != RSSL_RET_BLANK_DATA)
		{
			printf("rsslDecodeInt() failed with return code: %d\n", ret);
			return ret;
		}
		break;
	case RSSL_DT_FLOAT:
		if ((ret = rsslDecodeFloat(dIter, &fidFloatValue)) == RSSL_RET_SUCCESS)
		{
			printf("%f\n", fidFloatValue);
		}
		else if (ret != RSSL_RET_BLANK_DATA)
		{
			printf("rsslDecodeFloat() failed with return code: %d\n", ret);
			return ret;
		}
		break;
	case RSSL_DT_DOUBLE:
		if ((ret = rsslDecodeDouble(dIter, &fidDoubleValue)) == RSSL_RET_SUCCESS)
		{
			printf("%f\n", fidDoubleValue);
		}
		else if (ret != RSSL_RET_BLANK_DATA)
		{
			printf("rsslDecodeDouble() failed with return code: %d\n", ret);
			return ret;
		}
		break;
	case RSSL_DT_REAL:
		if ((ret = rsslDecodeReal(dIter, &fidRealValue)) == RSSL_RET_SUCCESS)
		{
			fidRealBuf.data = (char*)alloca(35);
			fidRealBuf.length = 35;
			rsslRealToString(&fidRealBuf, &fidRealValue);
			printf("%s\n", fidRealBuf.data);
		}
		else if (ret != RSSL_RET_BLANK_DATA)
		{
			printf("rsslDecodeReal() failed with return code: %d\n", ret);
			return ret;
		}
		break;
	case RSSL_DT_ENUM:
		if ((ret = rsslDecodeEnum(dIter, &fidEnumValue)) == RSSL_RET_SUCCESS)
		{
			RsslEnumType *pEnumType = getFieldEntryEnumType(dictionaryEntry, fidEnumValue);
			if (pEnumType)
				printf("%.*s(%d)\n", pEnumType->display.length, pEnumType->display.data, fidEnumValue);
			else
				printf("%d\n", fidEnumValue);
		}
		else if (ret != RSSL_RET_BLANK_DATA)
		{
			printf("rsslDecodeEnum() failed with return code: %d\n", ret);
			return ret;
		}
		break;
	case RSSL_DT_DATE:
		if ((ret = rsslDecodeDate(dIter, &fidDateTimeValue.date)) == RSSL_RET_SUCCESS)
		{
			fidDateTimeBuf.data = (char*)alloca(30);
			fidDateTimeBuf.length = 30;
			rsslDateTimeToString(&fidDateTimeBuf, RSSL_DT_DATE, &fidDateTimeValue);
			printf("%s\n", fidDateTimeBuf.data);
		}
		else if (ret != RSSL_RET_BLANK_DATA)
		{
			printf("rsslDecodeDate() failed with return code: %d\n", ret);
			return ret;
		}
		break;
	case RSSL_DT_TIME:
		if ((ret = rsslDecodeTime(dIter, &fidDateTimeValue.time)) == RSSL_RET_SUCCESS)
		{
			fidDateTimeBuf.data = (char*)alloca(30);
			fidDateTimeBuf.length = 30;
			rsslDateTimeToString(&fidDateTimeBuf, RSSL_DT_TIME, &fidDateTimeValue);
			printf("%s\n", fidDateTimeBuf.data);
		}
		else if (ret != RSSL_RET_BLANK_DATA)
		{
			printf("rsslDecodeTime() failed with return code: %d\n", ret);
			return ret;
		}
		break;
	case RSSL_DT_DATETIME:
		if ((ret = rsslDecodeDateTime(dIter, &fidDateTimeValue)) == RSSL_RET_SUCCESS)
		{
			fidDateTimeBuf.data = (char*)alloca(50);
			fidDateTimeBuf.length = 50;
			rsslDateTimeToString(&fidDateTimeBuf, RSSL_DT_DATETIME, &fidDateTimeValue);
			printf("%s\n", fidDateTimeBuf.data);
		}
		else if (ret != RSSL_RET_BLANK_DATA)
		{
			printf("rsslDecodeDateTime() failed with return code: %d\n", ret);
			return ret;
		}
		break;
	case RSSL_DT_QOS:
		if ((ret = rsslDecodeQos(dIter, &fidQosValue)) == RSSL_RET_SUCCESS) {
			fidQosBuf.data = (char*)alloca(100);
			fidQosBuf.length = 100;
			rsslQosToString(&fidQosBuf, &fidQosValue);
			printf("%s\n", fidQosBuf.data);
		}
		else if (ret != RSSL_RET_BLANK_DATA)
		{
			printf("rsslDecodeQos() failed with return code: %d\n", ret);
			return ret;
		}
		break;
	case RSSL_DT_STATE:
		if ((ret = rsslDecodeState(dIter, &fidStateValue)) == RSSL_RET_SUCCESS) {
			int stateBufLen = 80;
			if (fidStateValue.text.data)
				stateBufLen += fidStateValue.text.length;
			fidStateBuf.data = (char*)alloca(stateBufLen);
			fidStateBuf.length = stateBufLen;
			rsslStateToString(&fidStateBuf, &fidStateValue);
			printf("%.*s\n", fidStateBuf.length, fidStateBuf.data);
		}
		else if (ret != RSSL_RET_BLANK_DATA)
		{
			printf("rsslDecodeState() failed with return code: %d\n", ret);
			return ret;
		}
		break;

		/*For an example of array decoding, see fieldListEncDec.c*/
	case RSSL_DT_ARRAY:
		break;
	case RSSL_DT_BUFFER:
	case RSSL_DT_ASCII_STRING:
	case RSSL_DT_UTF8_STRING:
	case RSSL_DT_RMTES_STRING:
		if ((ret = rsslDecodeBuffer(dIter, &fidBufferValue)) == RSSL_RET_SUCCESS)
		{
			printf("%.*s\n", fidBufferValue.length, fidBufferValue.data);
		}
		else if (ret != RSSL_RET_BLANK_DATA)
		{
			printf("rsslDecodeBuffer() failed with return code: %d\n", ret);
			return ret;
		}
		break;
	default:
		printf("Unsupported data type (%d) for fid value\n", dataType);
		break;
	}
	if (ret == RSSL_RET_BLANK_DATA)
	{
		printf("<blank data>\n");
	}

	return RSSL_RET_SUCCESS;
}


